/* 
 * Crystal Mud Client
 * Copyright (C) 1998-2002 Abigail Brady
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>

#include "config.h"

#include "Socket.h"

Socket::Socket() {
  buffer = "";
  fd = -1;
}

int Socket::connect(const char *where, int port) {
    struct sockaddr_in addr;
    addr.sin_family = AF_INET;
    addr.sin_port = htons(port);
    inet_aton(where, &addr.sin_addr);
    
    fd = socket(AF_INET, SOCK_STREAM, 0);
    int value = ::connect(fd, (struct sockaddr *)&addr, sizeof addr);
    
    int er = errno;
    
    if (value == -1) {
	::close(fd);
	return -er;
    }
    
    fcntl(fd, F_SETFL, O_NONBLOCK);
    dead = false;
    
    return true;
}

int Socket::read(char *data, int size) {
  if (dead) {
      errno = ECONNRESET;
      return -1;
  }
  int rval = ::read(fd, data, size);
  if (rval == -1) {
      if (errno == EAGAIN || errno == EWOULDBLOCK) {
	  return -1;
      }
      dead = true;
  }
  return rval;
}

int Socket::write(const char *data, int size) {
    if (dead) {
	errno = ECONNRESET;
	return -1;
    }
    int rval = ::write(fd, data, size);
    if (rval == -1) {
	dead = true;
    }
    return rval;
}

Socket::~Socket() {
  if (!dead) {
    shutdown(fd, 2);
    ::close(fd);
  }
}

void Socket::close() {
  shutdown(fd, 2);
  ::close(fd);
  dead = true;
}

